@@ -46,7 +46,6 @@
 #include <memory>
 #include "CacheMemory.h"
 #include "Clip.h"
-#include "Exceptions.h"
 #include "OpenMPUtilities.h"
 #include "Settings.h"
 
@@ -76,11 +75,11 @@ namespace openshot {
 	 *
 	 * @code
 	 * // Create a reader for a video
-	 * FFmpegReader r("MyAwesomeVideo.webm");
+	 * openshot::FFmpegReader r("MyAwesomeVideo.webm");
 	 * r.Open(); // Open the reader
 	 *
 	 * // Get frame number 1 from the video
-	 * std::shared_ptr<Frame> f = r.GetFrame(1);
+	 * std::shared_ptr<openshot::Frame> f = r.GetFrame(1);
 	 *
 	 * // Now that we have an openshot::Frame object, lets have some fun!
 	 * f->Display(); // Display the frame on the screen
@@ -98,7 +97,7 @@ namespace openshot {
 		AVFormatContext *pFormatCtx;
 		int i, videoStream, audioStream;
 		AVCodecContext *pCodecCtx, *aCodecCtx;
-#if (LIBAVFORMAT_VERSION_MAJOR >= 57)
+#if USE_HW_ACCEL
 		AVBufferRef *hw_device_ctx = NULL; //PM
 #endif
 		AVStream *pStream, *aStream;
@@ -109,6 +108,7 @@ namespace openshot {
 		bool check_interlace;
 		bool check_fps;
 		bool has_missing_frames;
+		int max_concurrent_frames;
 
 		CacheMemory working_cache;
 		CacheMemory missing_frames;
@@ -147,12 +147,11 @@ namespace openshot {
 		int64_t current_video_frame;    // can't reliably use PTS of video to determine this
 
 		int hw_de_supported = 0;    // Is set by FFmpegReader
-#if IS_FFMPEG_3_2
+#if USE_HW_ACCEL
 		AVPixelFormat hw_de_av_pix_fmt = AV_PIX_FMT_NONE;
 		AVHWDeviceType hw_de_av_device_type = AV_HWDEVICE_TYPE_NONE;
-#endif
-
 		int IsHardwareDecodeSupported(int codecid);
+#endif
 
 		/// Check for the correct frames per second value by scanning the 1st few seconds of video packets.
 		void CheckFPS();
@@ -196,6 +195,9 @@ namespace openshot {
 		/// Get the PTS for the current video packet
 		int64_t GetVideoPTS();
 
+		/// Check if there's an album art
+		bool HasAlbumArt();
+
 		/// Remove partial frames due to seek
 		bool IsPartialFrame(int64_t requested_frame);
 
@@ -234,44 +236,46 @@ namespace openshot {
 		/// codecs have trouble seeking, and can introduce artifacts or blank images into the video.
 		bool enable_seek;
 
-		/// Constructor for FFmpegReader.  This automatically opens the media file and loads
-		/// frame 1, or it throws one of the following exceptions.
-		FFmpegReader(std::string path);
-
-		/// Constructor for FFmpegReader.  This only opens the media file to inspect its properties
-		/// if inspect_reader=true. When not inspecting the media file, it's much faster, and useful
-		/// when you are inflating the object using JSON after instantiating it.
-		FFmpegReader(std::string path, bool inspect_reader);
+		/// @brief Constructor for FFmpegReader.
+		///
+		/// Sets (and possibly opens) the media file path,
+		/// or throws an exception.
+		/// @param path  The filesystem location to load
+		/// @param inspect_reader  if true (the default), automatically open the media file and loads frame 1.
+		FFmpegReader(const std::string& path, bool inspect_reader=true);
 
 		/// Destructor
 		virtual ~FFmpegReader();
 
 		/// Close File
-		void Close();
+		void Close() override;
 
 		/// Get the cache object used by this reader
-		CacheMemory *GetCache() { return &final_cache; };
+		CacheMemory *GetCache() override { return &final_cache; };
 
 		/// Get a shared pointer to a openshot::Frame object for a specific frame number of this reader.
 		///
 		/// @returns The requested frame of video
 		/// @param requested_frame	The frame number that is requested.
-		std::shared_ptr<openshot::Frame> GetFrame(int64_t requested_frame);
+		std::shared_ptr<openshot::Frame> GetFrame(int64_t requested_frame) override;
 
 		/// Determine if reader is open or closed
-		bool IsOpen() { return is_open; };
+		bool IsOpen() override { return is_open; };
 
 		/// Return the type name of the class
-		std::string Name() { return "FFmpegReader"; };
+		std::string Name() override { return "FFmpegReader"; };
 
-		/// Get and Set JSON methods
-		std::string Json(); ///< Generate JSON string of this object
-		void SetJson(std::string value); ///< Load JSON string into this object
-		Json::Value JsonValue(); ///< Generate Json::JsonValue for this object
-		void SetJsonValue(Json::Value root); ///< Load Json::JsonValue into this object
+		// Get and Set JSON methods
+		std::string Json() const override; ///< Generate JSON string of this object
+		void SetJson(const std::string value) override; ///< Load JSON string into this object
+		Json::Value JsonValue() const override; ///< Generate Json::Value for this object
+		void SetJsonValue(const Json::Value root) override; ///< Load Json::Value into this object
 
 		/// Open File - which is called by the constructor automatically
-		void Open();
+		void Open() override;
+
+		/// Return true if frame can be read with GetFrame()
+		bool GetIsDurationKnown();
 	};
 
 }
